@@ -43,6 +43,7 @@
 
 #include "xf86Crtc.h"
 
+#include "compat-api.h"
 #ifdef USE_EXA
 #include "exa.h"
 #endif
@@ -55,6 +56,73 @@ extern DriverRec RADEON;
 #define RADEON_MAX_CRTC 6
 #define RADEON_MAX_BIOS_CONNECTOR 16
 
+typedef enum {
+    CHIP_FAMILY_UNKNOW,
+    CHIP_FAMILY_LEGACY,
+    CHIP_FAMILY_RADEON,
+    CHIP_FAMILY_RV100,
+    CHIP_FAMILY_RS100,    /* U1 (IGP320M) or A3 (IGP320)*/
+    CHIP_FAMILY_RV200,
+    CHIP_FAMILY_RS200,    /* U2 (IGP330M/340M/350M) or A4 (IGP330/340/345/350), RS250 (IGP 7000) */
+    CHIP_FAMILY_R200,
+    CHIP_FAMILY_RV250,
+    CHIP_FAMILY_RS300,    /* RS300/RS350 */
+    CHIP_FAMILY_RV280,
+    CHIP_FAMILY_R300,
+    CHIP_FAMILY_R350,
+    CHIP_FAMILY_RV350,
+    CHIP_FAMILY_RV380,    /* RV370/RV380/M22/M24 */
+    CHIP_FAMILY_R420,     /* R420/R423/M18 */
+    CHIP_FAMILY_RV410,    /* RV410, M26 */
+    CHIP_FAMILY_RS400,    /* xpress 200, 200m (RS400) Intel */
+    CHIP_FAMILY_RS480,    /* xpress 200, 200m (RS410/480/482/485) AMD */
+    CHIP_FAMILY_RV515,    /* rv515 */
+    CHIP_FAMILY_R520,    /* r520 */
+    CHIP_FAMILY_RV530,    /* rv530 */
+    CHIP_FAMILY_R580,    /* r580 */
+    CHIP_FAMILY_RV560,   /* rv560 */
+    CHIP_FAMILY_RV570,   /* rv570 */
+    CHIP_FAMILY_RS600,
+    CHIP_FAMILY_RS690,
+    CHIP_FAMILY_RS740,
+    CHIP_FAMILY_R600,    /* r600 */
+    CHIP_FAMILY_RV610,
+    CHIP_FAMILY_RV630,
+    CHIP_FAMILY_RV670,
+    CHIP_FAMILY_RV620,
+    CHIP_FAMILY_RV635,
+    CHIP_FAMILY_RS780,
+    CHIP_FAMILY_RS880,
+    CHIP_FAMILY_RV770,   /* r700 */
+    CHIP_FAMILY_RV730,
+    CHIP_FAMILY_RV710,
+    CHIP_FAMILY_RV740,
+    CHIP_FAMILY_CEDAR,   /* evergreen */
+    CHIP_FAMILY_REDWOOD,
+    CHIP_FAMILY_JUNIPER,
+    CHIP_FAMILY_CYPRESS,
+    CHIP_FAMILY_HEMLOCK,
+    CHIP_FAMILY_PALM,
+    CHIP_FAMILY_SUMO,
+    CHIP_FAMILY_SUMO2,
+    CHIP_FAMILY_BARTS,
+    CHIP_FAMILY_TURKS,
+    CHIP_FAMILY_CAICOS,
+    CHIP_FAMILY_CAYMAN,
+    CHIP_FAMILY_ARUBA,
+    CHIP_FAMILY_LAST
+} RADEONChipFamily;
+
+typedef struct {
+    uint32_t pci_device_id;
+    RADEONChipFamily chip_family;
+    int mobility;
+    int igp;
+    int nocrtc2;
+    int nointtvout;
+    int singledac;
+} RADEONCardInfo;
+
 typedef enum
 {
     MT_UNKNOWN = -1,
@@ -147,6 +215,11 @@ typedef struct
     Bool hw_capable;
 } RADEONI2CBusRec, *RADEONI2CBusPtr;
 
+enum radeon_pll_algo {
+    RADEON_PLL_OLD,
+    RADEON_PLL_NEW
+};
+
 typedef struct _RADEONCrtcPrivateRec {
     void *crtc_rotate_mem;
     void *cursor_mem;
@@ -164,6 +237,7 @@ typedef struct _RADEONCrtcPrivateRec {
     float vsc;
     float hsc;
     int pll_id;
+    enum radeon_pll_algo     pll_algo;
 } RADEONCrtcPrivateRec, *RADEONCrtcPrivatePtr;
 
 typedef struct _radeon_encoder {
@@ -338,9 +412,12 @@ struct avivo_crtc_state {
 struct avivo_grph_state {
     uint32_t enable;
     uint32_t control;
+    uint32_t swap_control;
     uint32_t prim_surf_addr;
     uint32_t sec_surf_addr;
     uint32_t pitch;
+    uint32_t prim_surf_addr_hi;
+    uint32_t sec_surf_addr_hi;
     uint32_t x_offset;
     uint32_t y_offset;
     uint32_t x_start;
@@ -354,6 +431,37 @@ struct avivo_grph_state {
     uint32_t mode_data_format;
 };
 
+struct dce4_main_block_state {
+    struct avivo_grph_state grph;
+    uint32_t scl[6];
+    uint32_t crtc[15];
+    uint32_t fmt[10];
+    uint32_t dig[20];
+};
+
+struct dce4_state
+{
+
+    uint32_t vga1_cntl;
+    uint32_t vga2_cntl;
+    uint32_t vga3_cntl;
+    uint32_t vga4_cntl;
+    uint32_t vga5_cntl;
+    uint32_t vga6_cntl;
+    uint32_t vga_render_control;
+
+    struct dce4_main_block_state block[6];
+
+    uint32_t vga_pll[3][3];
+    uint32_t pll[2][15];
+    uint32_t pll_route[6];
+
+    uint32_t dac[2][26];
+    uint32_t uniphy[6][10];
+
+    uint32_t dig[20];
+};
+
 struct avivo_state
 {
     uint32_t hdp_fb_location;
@@ -363,24 +471,25 @@ struct avivo_state
 
     uint32_t vga1_cntl;
     uint32_t vga2_cntl;
+    uint32_t vga3_cntl;
+    uint32_t vga4_cntl;
+    uint32_t vga5_cntl;
+    uint32_t vga6_cntl;
     uint32_t vga_render_control;
 
     uint32_t crtc_master_en;
     uint32_t crtc_tv_control;
     uint32_t dc_lb_memory_split;
 
-    struct avivo_pll_state pll1;
-    struct avivo_pll_state pll2;
+    struct avivo_pll_state pll[2];
 
     struct avivo_pll_state vga25_ppll;
     struct avivo_pll_state vga28_ppll;
     struct avivo_pll_state vga41_ppll;
 
-    struct avivo_crtc_state crtc1;
-    struct avivo_crtc_state crtc2;
+    struct avivo_crtc_state crtc[2];
 
-    struct avivo_grph_state grph1;
-    struct avivo_grph_state grph2;
+    struct avivo_grph_state grph[2];
 
     /* DDIA block on RS6xx chips */
     uint32_t ddia[37];
@@ -434,6 +543,7 @@ struct avivo_state
 
 typedef struct {
     struct avivo_state avivo;
+    struct dce4_state dce4;
 
 				/* Common registers */
     uint32_t          ovr_clr;
@@ -555,8 +665,8 @@ typedef struct {
 
 				/* Pallet */
     Bool              palette_valid;
-    uint32_t          palette[256];
-    uint32_t          palette2[256];
+    Bool	      palette_saved[2];
+    uint32_t          palette[2][256];
 
     uint32_t          disp2_req_cntl1;
     uint32_t          disp2_req_cntl2;
@@ -638,7 +748,12 @@ typedef struct
     RADEONSaveRec     SavedReg;         /* Original (text) mode              */
 
     void              *MMIO;            /* Map of MMIO region                */
+    int               MMIO_cnt;         /* Map of FB region refcount         */
+    void              *FB;              /* Map of FB region                  */
+    int               FB_cnt;           /* Map of FB region refcount         */
     int fd;                             /* for sharing across zaphod heads   */
+    unsigned long     fd_wakeup_registered; /* server generation for which fd has been registered for wakeup handling */
+    int dri2_info_cnt;
 } RADEONEntRec, *RADEONEntPtr;
 
 /* radeon_probe.c */
@@ -646,28 +761,28 @@ extern PciChipsets          RADEONPciChipsets[];
 
 /* radeon_driver.c */
 extern Bool                 RADEONPreInit(ScrnInfoPtr, int);
-extern Bool                 RADEONScreenInit(int, ScreenPtr, int, char **);
-extern Bool                 RADEONSwitchMode(int, DisplayModePtr, int);
+extern Bool                 RADEONScreenInit(SCREEN_INIT_ARGS_DECL);
+extern Bool                 RADEONSwitchMode(SWITCH_MODE_ARGS_DECL);
 #ifdef X_XF86MiscPassMessage
 extern Bool                 RADEONHandleMessage(int, const char*, const char*,
 					        char**);
 #endif
-extern void                 RADEONAdjustFrame(int, int, int, int);
-extern Bool                 RADEONEnterVT(int, int);
-extern void                 RADEONLeaveVT(int, int);
-extern void                 RADEONFreeScreen(int, int);
-extern ModeStatus           RADEONValidMode(int, DisplayModePtr, Bool, int);
+extern void                 RADEONAdjustFrame(ADJUST_FRAME_ARGS_DECL);
+extern Bool                 RADEONEnterVT(VT_FUNC_ARGS_DECL);
+extern void                 RADEONLeaveVT(VT_FUNC_ARGS_DECL);
+extern void                 RADEONFreeScreen(FREE_SCREEN_ARGS_DECL);
+extern ModeStatus           RADEONValidMode(SCRN_ARG_TYPE, DisplayModePtr, Bool, int);
 
 extern const OptionInfoRec *RADEONOptionsWeak(void);
 
 #ifdef XF86DRM_MODE
 extern Bool                 RADEONPreInit_KMS(ScrnInfoPtr, int);
-extern Bool                 RADEONScreenInit_KMS(int, ScreenPtr, int, char **);
-extern Bool                 RADEONSwitchMode_KMS(int, DisplayModePtr, int);
-extern void                 RADEONAdjustFrame_KMS(int, int, int, int);
-extern Bool                 RADEONEnterVT_KMS(int, int);
-extern void                 RADEONLeaveVT_KMS(int, int);
-extern void RADEONFreeScreen_KMS(int scrnIndex, int flags);
+extern Bool                 RADEONScreenInit_KMS(SCREEN_INIT_ARGS_DECL);
+extern Bool                 RADEONSwitchMode_KMS(SWITCH_MODE_ARGS_DECL);
+extern void                 RADEONAdjustFrame_KMS(ADJUST_FRAME_ARGS_DECL);
+extern Bool                 RADEONEnterVT_KMS(VT_FUNC_ARGS_DECL);
+extern void                 RADEONLeaveVT_KMS(VT_FUNC_ARGS_DECL);
+extern void RADEONFreeScreen_KMS(FREE_SCREEN_ARGS_DECL);
 #endif
 
 #endif /* _RADEON_PROBE_H_ */
